@@ -142,7 +142,7 @@ if ($_SERVER['REQUEST_METHOD'] == "POST") {
     }
 
     if (isset($choice_of_reply[$tInterval_Time])) {
-        $fInterval_Time = $tInterval_Time;
+        $fInterval_Time = (int) $tInterval_Time;
     } else {
         $fInterval_Time = 0;
     }
@@ -154,7 +154,7 @@ if ($_SERVER['REQUEST_METHOD'] == "POST") {
 
     if ($action == 'fChange') {
 
-        ## check if ActiveUnitl is not  back in time,
+        ## check if ActiveUntil is not  back in time,
         ## because vacation.pl will report SMTP recipient $smtp_recipient which resolves to $email does not have an active vacation (rv: $rv, email: $email)"
         ## and will not send message
 
